/* Container for all chatboxes */
.sage-ai-chatbox-container {
  display: flex;
  flex-direction: column;
  height: 100%;
  width: 100%;
  overflow: visible; /* Allow dropdowns to overflow */
  position: relative; /* Added for dropdown positioning */
}

/* Individual chatboxes */
.sage-ai-notebook-chatbox {
  flex: 1;
  display: flex;
  flex-direction: column;
  height: 100%;
  width: 100%;
}

/* Hidden chatbox */
.hidden-chatbox {
  display: none !important;
  visibility: hidden;
}

/* Ensure only one chatbox is visible at a time */
.sage-ai-notebook-chatbox:not(.hidden-chatbox) {
  display: flex !important;
}



/* Context cells indicator */
.context-cells-indicator {
  display: flex;
  align-items: center;
  font-size: 12px;
  padding: 8px 16px;
  margin: 0 0 8px 0;
  border-radius: 4px;
  background-color: #f0f0f0;
  color: #666;
  width: calc(100% - 32px);
}

.context-cells-indicator.has-context {
  background-color: #e6f7ff;
  color: #0366d6;
}

/* Hide indicator when no cells are in context */
.context-cells-indicator.hidden {
  display: none;
}

/* History container - added based on change description */
.sage-ai-history-container {
  overflow: visible; /* Allow dropdowns to overflow */
}
